/*
 * Gnpinpout.cpp
 *
 *  Created on: Jan 5, 2011
 *      Author: robert goerke
 */

#include <getopt.h>
#include <math.h>
#include <cstdlib>
#include <vector>
#include <exception>
#include <iostream>
#include <string>

#include "./GnpinpoutGenerator.h"

#ifndef NDEBUG
#define VERBOSE(x) x
#else
#define VERBOSE(x)
#endif

#define MAX(a,b) ((a>b) ? a:b)

using namespace boost;

int main(int argc, char **argv) {

	//default values
	bool create_boost_graph = false;
	int n = 1000;
	double pin_global = 0.3;
	double pout = 0.1;
	int k = 32;
	double skew_exponent = 1.0;
	bool randomize_k = false;
	std::string graph_out_file_name = "";
	std::string file_mode = "a";

	std::string cluster_sizes_in_file_name = "";
	bool use_cluster_sizes_in_file = false;
	std::string pin_in_file_name = "";
	bool use_pin_in_file = false;
	bool weighted = false;
	bool verbosity = false;
	bool report = true;
	int random_add = 0;

	//	  std::cout << sizeof(long long) << "\n";

	if (argc == 1) {
		std::cout << "usage: Gnpinpout" << "\n"
				<< " \t-n <int> number of nodes, default is " << n << "\n"
				<< " \t-k <int> number of clusters, default is " << k << "\n"
				<< " \t-c <string> textfile containing individual cluster sizes (overrides k,r), default is not to use one" << "\n"
				<< " \t-s <double> skewness of cluster sizes (exp. of random number), default is " << skew_exponent << "\n"
				<< " \t-r flag, randomize number of clusters uniformly between 1...2*sqrt(n), default is " << randomize_k << "\n"
				<< " \t-x <double> external edge probability, default is " << pout	<< "\n"
				<< " \t-i <double> internal edge probability, default is " << pin_global << "\n"
				<< " \t-p <string> textfile containing individual internal edge probabilities, default is not to use one" << "\n"
				<< " \t-f <string> file mode, a=ascii, b=binary, n=none, default is " << file_mode << "\n"
				<< " \t-o <string> the output graph file name, default is a timestamp\n"
				<< " \t-w flag, weigh edges uniformly between [0..1), default is " << weighted << "\n"
		//				<< " \t-b flag, internally create a boost graph, default is " << create_boost_graph << "\n"
				<< "\t-R <int> a number to be added to the seed of the generator e.g. from /dev/random via $RANDOM (important for script usage)" << "\n"
				<< " \t-v flag, be verbose, default is " << verbosity << "\n"
				<< " \t-q flag, be quiet, default is " << !report << "\n";
		//		std::cout << "...generating anayway, using defaults" << "\n";
		return 0;
	}

	// read arguments
	int c;
	while ((c = getopt(argc, argv, "n:k:s:rx:i:p:c:f:o:wb:vqR:")) != -1) { //ez
		switch (c) {
		case 'n':
			n = atoi(optarg);
			//                    	  n = std::min(std::max(n,0),100000);
			break;
		case 'k':
			k = atoi(optarg);
			k = std::min(MAX(k,0), n);
			break;
		case 'c':
			cluster_sizes_in_file_name = optarg;
			use_cluster_sizes_in_file = true;
			break;
		case 's':
			skew_exponent = atof(optarg);
			//                    	  skew_exponent = std::min(std::max(skew_exponent,0.0),10.0);
			break;
		case 'r':
			randomize_k = true;
			break;
		case 'x':
			pout = atof(optarg);
			pout = std::min(std::max(pout, 0.0), 1.0);
			break;
		case 'i':
			pin_global = atof(optarg);
			pin_global = std::min(std::max(pin_global, 0.0), 1.0);
			break;
		case 'p':
			pin_in_file_name = optarg;
			use_pin_in_file = true;
			break;
		case 'f':
			file_mode = optarg;
			break;
		case 'o':
			graph_out_file_name = optarg;
			break;
		case 'w':
			weighted = true;
			break;
		case 'b':
			create_boost_graph = true;
			break;
		case 'v':
			verbosity = true;
			break;
		case 'q':
			report = false;
			verbosity = false;
			break;
		case 'R':
			random_add = atoi(optarg);
			break;
		}
	}

	//calling a generator
	GnpinpoutGenerator my_gen(random_add);
	if (report)
		std::cout << "generating now " << "\n";
//	graph_out_file_name = "1";
	my_gen.generate(n, k, skew_exponent, randomize_k, pout, pin_global, pin_in_file_name, cluster_sizes_in_file_name, weighted, verbosity, report);
	if (report)
		std::cout << "storing in \"" << graph_out_file_name << "\", using format " << file_mode << "\n";
	my_gen.store_graph_in_file(graph_out_file_name, file_mode);

	//do it again?
	//	graph_out_file_name = "2";
	//	my_gen.generate(n,k, skew_exponent, randomize_k, pout, pin_global, pin_in_file_name, cluster_sizes_in_file_name, weighted, verbosity, report);
	//	my_gen.store_graph_in_file(graph_out_file_name, file_mode);

	//create a boost graph and have it returned?
	//	// create a typedef for the Graph type
	//	typedef adjacency_list<listS, vecS, bidirectionalS, no_property, property<edge_weight_t, double> > GraphW;
	//	typedef adjacency_list<vecS, vecS, bidirectionalS> Graph;
	//
	//	if (create_boost_graph){
	//		GraphW *graphP = my_gen.get_boost_graph<GraphW>();
	//		//you must destroy graphP!
	//		delete graphP;
	//	}

	// get the edge array


	if (report)
		std::cout << "caller finished. " << "\n";
	return 0;
}
